Represents a single solution to a unification problem.
Summary
Types
@type t() :: %ShotUn.UnifSolution{ flex_pairs: [{ShotDs.Data.Term.term_id(), ShotDs.Data.Term.term_id()}], substitutions: [ShotDs.Data.Substitution.t()] }
The type of a pre-unification solution.
This includes a list of substitutions as well as unresolved flex-flex pairs which can be understood as future unification constraints.